@@ -75,6 +75,10 @@ public class Increment {
         return compactAfter;
     }
 
+    public boolean isEmpty() {
+        return newFiles.isEmpty() && compactBefore.isEmpty() && compactAfter.isEmpty();
+    }
+
     @Override
     public boolean equals(Object o) {
         if (this == o) {
@@ -93,4 +97,16 @@ public class Increment {
     public int hashCode() {
         return Objects.hash(newFiles, compactBefore, compactAfter);
     }
+
+    @Override
+    public String toString() {
+        return "Increment{"
+                + "newFiles="
+                + newFiles
+                + ", compactBefore="
+                + compactBefore
+                + ", compactAfter="
+                + compactAfter
+                + '}';
+    }
 }
