@@ -19,17 +19,37 @@
 package org.apache.flink.table.store.file.operation;
 
 import org.apache.flink.table.data.binary.BinaryRowData;
-import org.apache.flink.table.store.file.utils.RecordWriter;
+import org.apache.flink.table.store.file.compact.CompactResult;
+import org.apache.flink.table.store.file.data.DataFileMeta;
+import org.apache.flink.table.store.file.writer.RecordWriter;
 
+import javax.annotation.Nullable;
+
+import java.util.List;
+import java.util.concurrent.Callable;
 import java.util.concurrent.ExecutorService;
 
-/** Write operation which provides {@link RecordWriter} creation. */
-public interface FileStoreWrite {
+/**
+ * Write operation which provides {@link RecordWriter} creation.
+ *
+ * @param <T> type of record to write.
+ */
+public interface FileStoreWrite<T> {
 
     /** Create a {@link RecordWriter} from partition and bucket. */
-    RecordWriter createWriter(BinaryRowData partition, int bucket, ExecutorService compactExecutor);
+    RecordWriter<T> createWriter(
+            BinaryRowData partition, int bucket, ExecutorService compactExecutor);
 
     /** Create an empty {@link RecordWriter} from partition and bucket. */
-    RecordWriter createEmptyWriter(
+    RecordWriter<T> createEmptyWriter(
             BinaryRowData partition, int bucket, ExecutorService compactExecutor);
+
+    /**
+     * Create a {@link Callable} compactor from partition, bucket.
+     *
+     * @param compactFiles input files of compaction. When it is null, will automatically read all
+     *     files of the current bucket.
+     */
+    Callable<CompactResult> createCompactWriter(
+            BinaryRowData partition, int bucket, @Nullable List<DataFileMeta> compactFiles);
 }
