@@ -25,24 +25,29 @@ import org.apache.flink.core.fs.Path;
 import org.apache.flink.table.data.GenericRowData;
 import org.apache.flink.table.data.RowData;
 import org.apache.flink.table.data.binary.BinaryRowDataUtil;
+import org.apache.flink.table.store.CoreOptions;
+import org.apache.flink.table.store.CoreOptions.ChangelogProducer;
 import org.apache.flink.table.store.file.KeyValue;
-import org.apache.flink.table.store.file.ValueKind;
-import org.apache.flink.table.store.file.format.FileFormat;
+import org.apache.flink.table.store.file.data.DataFileMeta;
+import org.apache.flink.table.store.file.data.DataFileReader;
+import org.apache.flink.table.store.file.data.DataFileWriter;
 import org.apache.flink.table.store.file.format.FlushingFileFormat;
-import org.apache.flink.table.store.file.mergetree.compact.CompactManager;
+import org.apache.flink.table.store.file.memory.HeapMemorySegmentPool;
+import org.apache.flink.table.store.file.mergetree.compact.CompactRewriter;
 import org.apache.flink.table.store.file.mergetree.compact.CompactStrategy;
 import org.apache.flink.table.store.file.mergetree.compact.DeduplicateMergeFunction;
 import org.apache.flink.table.store.file.mergetree.compact.IntervalPartition;
+import org.apache.flink.table.store.file.mergetree.compact.MergeTreeCompactManager;
 import org.apache.flink.table.store.file.mergetree.compact.UniversalCompaction;
-import org.apache.flink.table.store.file.mergetree.sst.SstFileMeta;
-import org.apache.flink.table.store.file.mergetree.sst.SstFileReader;
-import org.apache.flink.table.store.file.mergetree.sst.SstFileWriter;
+import org.apache.flink.table.store.file.schema.SchemaManager;
 import org.apache.flink.table.store.file.utils.FileStorePathFactory;
 import org.apache.flink.table.store.file.utils.RecordReader;
 import org.apache.flink.table.store.file.utils.RecordReaderIterator;
-import org.apache.flink.table.store.file.utils.RecordWriter;
+import org.apache.flink.table.store.file.writer.RecordWriter;
+import org.apache.flink.table.store.format.FileFormat;
 import org.apache.flink.table.types.logical.IntType;
 import org.apache.flink.table.types.logical.RowType;
+import org.apache.flink.types.RowKind;
 
 import org.junit.jupiter.api.AfterAll;
 import org.junit.jupiter.api.BeforeAll;
@@ -74,42 +79,51 @@ public class MergeTreeTest {
 
     @TempDir java.nio.file.Path tempDir;
     private static ExecutorService service;
+    private Path path;
     private FileStorePathFactory pathFactory;
     private Comparator<RowData> comparator;
 
-    private MergeTreeOptions options;
-    private SstFileReader sstFileReader;
-    private SstFileWriter sstFileWriter;
-    private RecordWriter writer;
+    private CoreOptions options;
+    private DataFileReader dataFileReader;
+    private DataFileWriter dataFileWriter;
+    private RecordWriter<KeyValue> writer;
 
     @BeforeEach
     public void beforeEach() throws IOException {
-        pathFactory = new FileStorePathFactory(new Path(tempDir.toString()));
+        path = new Path(tempDir.toString());
+        pathFactory = new FileStorePathFactory(path);
         comparator = Comparator.comparingInt(o -> o.getInt(0));
         recreateMergeTree(1024 * 1024);
-        Path bucketDir = sstFileWriter.pathFactory().toPath("ignore").getParent();
+        Path bucketDir = dataFileWriter.pathFactory().toPath("ignore").getParent();
         bucketDir.getFileSystem().mkdirs(bucketDir);
     }
 
     private void recreateMergeTree(long targetFileSize) {
         Configuration configuration = new Configuration();
-        configuration.set(MergeTreeOptions.WRITE_BUFFER_SIZE, new MemorySize(4096 * 3));
-        configuration.set(MergeTreeOptions.PAGE_SIZE, new MemorySize(4096));
-        configuration.set(MergeTreeOptions.TARGET_FILE_SIZE, new MemorySize(targetFileSize));
-        options = new MergeTreeOptions(configuration);
+        configuration.set(CoreOptions.WRITE_BUFFER_SIZE, new MemorySize(4096 * 3));
+        configuration.set(CoreOptions.PAGE_SIZE, new MemorySize(4096));
+        configuration.set(CoreOptions.TARGET_FILE_SIZE, new MemorySize(targetFileSize));
+        options = new CoreOptions(configuration);
         RowType keyType = new RowType(singletonList(new RowType.RowField("k", new IntType())));
         RowType valueType = new RowType(singletonList(new RowType.RowField("v", new IntType())));
         FileFormat flushingAvro = new FlushingFileFormat("avro");
-        sstFileReader =
-                new SstFileReader.Factory(keyType, valueType, flushingAvro, pathFactory)
+        dataFileReader =
+                new DataFileReader.Factory(
+                                new SchemaManager(path),
+                                0,
+                                keyType,
+                                valueType,
+                                flushingAvro,
+                                pathFactory)
                         .create(BinaryRowDataUtil.EMPTY_ROW, 0);
-        sstFileWriter =
-                new SstFileWriter.Factory(
+        dataFileWriter =
+                new DataFileWriter.Factory(
+                                0,
                                 keyType,
                                 valueType,
                                 flushingAvro,
                                 pathFactory,
-                                options.targetFileSize)
+                                options.targetFileSize())
                         .create(BinaryRowDataUtil.EMPTY_ROW, 0);
         writer = createMergeTreeWriter(Collections.emptyList());
     }
@@ -153,10 +167,10 @@ public class MergeTreeTest {
     @Test
     public void testRestore() throws Exception {
         List<TestRecord> expected = new ArrayList<>(writeBatch());
-        List<SstFileMeta> newFiles = writer.prepareCommit().newFiles();
+        List<DataFileMeta> newFiles = writer.prepareCommit(true).newFiles();
         writer = createMergeTreeWriter(newFiles);
         expected.addAll(writeBatch());
-        writer.prepareCommit();
+        writer.prepareCommit(true);
         writer.sync();
         assertRecords(expected);
     }
@@ -164,9 +178,9 @@ public class MergeTreeTest {
     @Test
     public void testClose() throws Exception {
         doTestWriteRead(6);
-        List<SstFileMeta> files = writer.close();
-        for (SstFileMeta file : files) {
-            Path path = sstFileWriter.pathFactory().toPath(file.fileName());
+        List<DataFileMeta> files = writer.close();
+        for (DataFileMeta file : files) {
+            Path path = dataFileWriter.pathFactory().toPath(file.fileName());
             assertThat(path.getFileSystem().exists(path)).isFalse();
         }
     }
@@ -181,19 +195,19 @@ public class MergeTreeTest {
         Random random = new Random();
         int perBatch = 1_000;
         Set<String> newFileNames = new HashSet<>();
-        List<SstFileMeta> compactedFiles = new ArrayList<>();
+        List<DataFileMeta> compactedFiles = new ArrayList<>();
         for (int i = 0; i < 6; i++) {
             List<TestRecord> records = new ArrayList<>(perBatch);
             for (int j = 0; j < perBatch; j++) {
                 records.add(
                         new TestRecord(
-                                random.nextBoolean() ? ValueKind.ADD : ValueKind.DELETE,
+                                random.nextBoolean() ? RowKind.INSERT : RowKind.DELETE,
                                 random.nextInt(perBatch / 2) - i * (perBatch / 2),
                                 random.nextInt()));
             }
             writeAll(records);
             expected.addAll(records);
-            Increment increment = writer.prepareCommit();
+            Increment increment = writer.prepareCommit(true);
             mergeCompacted(newFileNames, compactedFiles, increment);
         }
         writer.close();
@@ -212,9 +226,9 @@ public class MergeTreeTest {
 
     private void doTestWriteRead(int batchNumber, int perBatch) throws Exception {
         List<TestRecord> expected = new ArrayList<>();
-        List<SstFileMeta> newFiles = new ArrayList<>();
+        List<DataFileMeta> newFiles = new ArrayList<>();
         Set<String> newFileNames = new HashSet<>();
-        List<SstFileMeta> compactedFiles = new ArrayList<>();
+        List<DataFileMeta> compactedFiles = new ArrayList<>();
 
         // write batch and commit
         for (int i = 0; i <= batchNumber; i++) {
@@ -224,7 +238,7 @@ public class MergeTreeTest {
                 writer.sync();
             }
 
-            Increment increment = writer.prepareCommit();
+            Increment increment = writer.prepareCommit(true);
             newFiles.addAll(increment.newFiles());
             mergeCompacted(newFileNames, compactedFiles, increment);
         }
@@ -239,72 +253,82 @@ public class MergeTreeTest {
         // assert records from increment compacted files
         assertRecords(expected, compactedFiles, true);
 
-        Path bucketDir = sstFileWriter.pathFactory().toPath("ignore").getParent();
+        Path bucketDir = dataFileWriter.pathFactory().toPath("ignore").getParent();
         Set<String> files =
                 Arrays.stream(bucketDir.getFileSystem().listStatus(bucketDir))
                         .map(FileStatus::getPath)
                         .map(Path::getName)
                         .collect(Collectors.toSet());
-        newFiles.stream().map(SstFileMeta::fileName).forEach(files::remove);
-        compactedFiles.stream().map(SstFileMeta::fileName).forEach(files::remove);
+        newFiles.stream().map(DataFileMeta::fileName).forEach(files::remove);
+        compactedFiles.stream().map(DataFileMeta::fileName).forEach(files::remove);
         assertThat(files).isEqualTo(Collections.emptySet());
     }
 
-    private MergeTreeWriter createMergeTreeWriter(List<SstFileMeta> files) {
+    private MergeTreeWriter createMergeTreeWriter(List<DataFileMeta> files) {
         long maxSequenceNumber =
-                files.stream().map(SstFileMeta::maxSequenceNumber).max(Long::compare).orElse(-1L);
-        return new MergeTreeWriter(
-                new SortBufferMemTable(
-                        sstFileWriter.keyType(),
-                        sstFileWriter.valueType(),
-                        options.writeBufferSize,
-                        options.pageSize),
-                createCompactManager(sstFileWriter, service),
-                new Levels(comparator, files, options.numLevels),
-                maxSequenceNumber,
-                comparator,
-                new DeduplicateMergeFunction(),
-                sstFileWriter,
-                options.commitForceCompact,
-                options.numSortedRunStopTrigger);
+                files.stream().map(DataFileMeta::maxSequenceNumber).max(Long::compare).orElse(-1L);
+        MergeTreeWriter writer =
+                new MergeTreeWriter(
+                        dataFileWriter.keyType(),
+                        dataFileWriter.valueType(),
+                        createCompactManager(dataFileWriter, service, files),
+                        new Levels(comparator, files, options.numLevels()),
+                        maxSequenceNumber,
+                        comparator,
+                        new DeduplicateMergeFunction(),
+                        dataFileWriter,
+                        options.commitForceCompact(),
+                        options.numSortedRunStopTrigger(),
+                        ChangelogProducer.NONE);
+        writer.setMemoryPool(
+                new HeapMemorySegmentPool(options.writeBufferSize(), options.pageSize()));
+        return writer;
     }
 
-    private CompactManager createCompactManager(
-            SstFileWriter sstFileWriter, ExecutorService compactExecutor) {
-        CompactStrategy compactStrategy =
+    private MergeTreeCompactManager createCompactManager(
+            DataFileWriter dataFileWriter,
+            ExecutorService compactExecutor,
+            List<DataFileMeta> files) {
+        CompactStrategy strategy =
                 new UniversalCompaction(
-                        options.maxSizeAmplificationPercent,
-                        options.sizeRatio,
-                        options.numSortedRunCompactionTrigger);
-        CompactManager.Rewriter rewriter =
+                        options.maxSizeAmplificationPercent(),
+                        options.sortedRunSizeRatio(),
+                        options.numSortedRunCompactionTrigger(),
+                        options.maxSortedRunNum());
+        CompactRewriter rewriter =
                 (outputLevel, dropDelete, sections) ->
-                        sstFileWriter.write(
-                                new RecordReaderIterator(
+                        dataFileWriter.write(
+                                new RecordReaderIterator<>(
                                         new MergeTreeReader(
                                                 sections,
                                                 dropDelete,
-                                                sstFileReader,
+                                                dataFileReader,
                                                 comparator,
                                                 new DeduplicateMergeFunction())),
                                 outputLevel);
-        return new CompactManager(
-                compactExecutor, compactStrategy, comparator, options.targetFileSize, rewriter);
+        return new MergeTreeCompactManager(
+                compactExecutor,
+                new Levels(comparator, files, options.numLevels()),
+                strategy,
+                comparator,
+                options.targetFileSize(),
+                rewriter);
     }
 
     private void mergeCompacted(
-            Set<String> newFileNames, List<SstFileMeta> compactedFiles, Increment increment) {
-        increment.newFiles().stream().map(SstFileMeta::fileName).forEach(newFileNames::add);
+            Set<String> newFileNames, List<DataFileMeta> compactedFiles, Increment increment) {
+        increment.newFiles().stream().map(DataFileMeta::fileName).forEach(newFileNames::add);
         compactedFiles.addAll(increment.newFiles());
         Set<String> afterFiles =
                 increment.compactAfter().stream()
-                        .map(SstFileMeta::fileName)
+                        .map(DataFileMeta::fileName)
                         .collect(Collectors.toSet());
-        for (SstFileMeta file : increment.compactBefore()) {
+        for (DataFileMeta file : increment.compactBefore()) {
             boolean remove = compactedFiles.remove(file);
             assertThat(remove).isTrue();
             // See MergeTreeWriter.updateCompactResult
             if (!newFileNames.contains(file.fileName()) && !afterFiles.contains(file.fileName())) {
-                sstFileWriter.delete(file);
+                dataFileWriter.delete(file);
             }
         }
         compactedFiles.addAll(increment.compactAfter());
@@ -322,12 +346,12 @@ public class MergeTreeTest {
 
     private void assertRecords(List<TestRecord> expected) throws Exception {
         // compaction will drop delete
-        List<SstFileMeta> files = ((MergeTreeWriter) writer).levels().allFiles();
+        List<DataFileMeta> files = ((MergeTreeWriter) writer).levels().allFiles();
         assertRecords(expected, files, true);
     }
 
     private void assertRecords(
-            List<TestRecord> expected, List<SstFileMeta> files, boolean dropDelete)
+            List<TestRecord> expected, List<DataFileMeta> files, boolean dropDelete)
             throws Exception {
         assertThat(readAll(files, dropDelete)).isEqualTo(compactAndSort(expected, dropDelete));
     }
@@ -339,7 +363,7 @@ public class MergeTreeTest {
         }
         if (dropDelete) {
             return map.values().stream()
-                    .filter(record -> record.kind == ValueKind.ADD)
+                    .filter(record -> record.kind == RowKind.INSERT)
                     .collect(Collectors.toList());
         }
         return new ArrayList<>(map.values());
@@ -347,20 +371,22 @@ public class MergeTreeTest {
 
     private void writeAll(List<TestRecord> records) throws Exception {
         for (TestRecord record : records) {
-            writer.write(record.kind, row(record.k), row(record.v));
+            KeyValue kv = new KeyValue().replace(row(record.k), record.kind, row(record.v));
+            writer.write(kv);
         }
     }
 
-    private List<TestRecord> readAll(List<SstFileMeta> files, boolean dropDelete) throws Exception {
-        RecordReader reader =
+    private List<TestRecord> readAll(List<DataFileMeta> files, boolean dropDelete)
+            throws Exception {
+        RecordReader<KeyValue> reader =
                 new MergeTreeReader(
                         new IntervalPartition(files, comparator).partition(),
                         dropDelete,
-                        sstFileReader,
+                        dataFileReader,
                         comparator,
                         new DeduplicateMergeFunction());
         List<TestRecord> records = new ArrayList<>();
-        try (RecordReaderIterator iterator = new RecordReaderIterator(reader)) {
+        try (RecordReaderIterator<KeyValue> iterator = new RecordReaderIterator<KeyValue>(reader)) {
             while (iterator.hasNext()) {
                 KeyValue kv = iterator.next();
                 records.add(
@@ -380,7 +406,7 @@ public class MergeTreeTest {
         for (int i = 0; i < perBatch; i++) {
             records.add(
                     new TestRecord(
-                            random.nextBoolean() ? ValueKind.ADD : ValueKind.DELETE,
+                            random.nextBoolean() ? RowKind.INSERT : RowKind.DELETE,
                             random.nextInt(perBatch / 2),
                             random.nextInt()));
         }
@@ -389,11 +415,11 @@ public class MergeTreeTest {
 
     private static class TestRecord {
 
-        private final ValueKind kind;
+        private final RowKind kind;
         private final int k;
         private final int v;
 
-        private TestRecord(ValueKind kind, int k, int v) {
+        private TestRecord(RowKind kind, int k, int v) {
             this.kind = kind;
             this.k = k;
             this.v = v;
